@@ -1,3 +1,11 @@
+#![deny(warnings)]
+#![deny(missing_docs)]
+
+//! # Symbols
+//! 
+//! This is an utility to build a proc-macro that connects to a database, retrieves data from given table and populates an enum variants with primary keys values.
+//! It also generates a method for every non-primary-key field, and, when there are multiple primary keys, a costructor for every possible subset of primary keys.
+
 use std::{collections::HashMap, env, fs, future::Future, io};
 
 use heck::{ToSnakeCase, ToUpperCamelCase};
@@ -14,12 +22,26 @@ use sea_orm::{
 
 use serde::{de::DeserializeOwned, Serialize};
 
-use symbols_models::EntityFilter;
+pub use symbols_models::EntityFilter;
 
 use syn::{
     punctuated::Punctuated, token::Comma, Fields, ItemEnum, Lit, LitBool, Meta, NestedMeta, Variant,
 };
 
+/// Main function
+/// Given a database model (via generics), an enum item, a list of arguments and an async function to retrieve a database connection
+/// it populates the enum using primary key(s) values.
+/// Only string-typed primary keys are supported.
+/// 
+/// When a single primary key is present, it simply generate an as_str method and a TryFrom<&str> implementation.
+/// When multiple primary keys are present, it generates a costructor for every possible subset of primary keys.
+/// 
+/// For every non-primary key field of a supported type, it generates a const method to retrieve it.
+/// 
+/// Replacements can be done on every string-typed field, even primary keys, and are done using annotated parameters.
+/// Two type of replacements are supported:
+/// * basic: written in the form #[macro(field = "enum")] or #[macro(field(type = "enum"))], where we are telling to replace string values from `field` with variants from enum `enum`, variant names will be the CamelCase version of field value.
+/// * advanced: written in the form #[macro(field(type = "bar", fn = "foo"))], where we are telling to replace string values from `field` with a call to method `foo` from struct/enum `bar`, method output is expected to be of type `bar`.
 pub async fn symbols<M, F, Fut>(
     item: &mut ItemEnum,
     args: &[NestedMeta],
@@ -394,11 +416,14 @@ where
     })
 }
 
+/// Replacement types
 enum Replacement {
     Type(TokenStream),
     Fn(TokenStream, Option<TokenStream>),
 }
 
+/// Field replacement facility
+/// Searches between macro arguments
 fn get_replacement<M>(col: M::Column, args: &[NestedMeta]) -> Option<Replacement>
 where
     M: EntityTrait,
@@ -447,7 +472,8 @@ where
     })
 }
 
-// file access is sync to not have to depend on an async runtime
+/// Data retrieve function with cache capabilities
+/// File access is sync to not have to depend on an async runtime
 async fn get_data<M, F, Fut>(get_conn: F) -> syn::Result<Vec<M::Model>>
 where
     M: EntityTrait + EntityFilter + Default,
